ArrTensor

Struct ArrTensor 

pub struct ArrTensor<T, const N: usize, const D: usize, const LANES: usize = MAX_SIMD_SINGLE_PRECISION_LANES>{ /* private fields */ }
A tensor made up of statically sized arrays.

Often the best choice for embedded tensor operations because it doesn’t use any OS-dependent features like heap allocators. If memory efficiency is the largest concern, the lack of dynamic heap allocation is a huge positive of ArrTensor.

However, when flexibility is put before memory efficiency and performance, this becomes obsolete; use DynTensor instead.

Implementations

Source

impl<T, const N: usize, const D: usize, const LANES: usize> ArrTensor<T, N, D, LANES>

pub fn new(shape: [usize; D]) -> Self
where T: Default + Copy,

Instantiates a new Tensor which owns its data without allocating it.

Panics

This constructor panics when the product of each dimension is not equal to the length of all the data.

pub fn with_data(shape: [usize; D], data: [T; N]) -> Self

Instantiates a new Tensor with data.

Panics

This constructor panics when the product of each dimension is not equal to the length of all the data.

Example:

use tensor_optim::ArrTensor;

const SHAPE: [usize; 2] = [2, 3];

let data = [0f64; 6];
let mut tensor = ArrTensor::with_data(SHAPE, data);

tensor += 42.0;
println!("First element: {}", tensor[&[0, 0]]);
Source

impl<T, const N: usize, const D: usize, const LANES: usize> ArrTensor<T, N, D, LANES>

pub fn map<U, F>(&self, f: F) -> ArrTensor<U, N, D, LANES>
where F: FnMut(T) -> U, U: SimdElement + Primitive, [U; LANES]: AlignedSimd<[U; LANES], U, LANES>,

Map each element of ArrTensor<T, N, D, LANES> to ArrTensor<U, N, D> by applying f elementwise.

pub fn zip_map<U, V, F>( &self, other: &ArrTensor<U, N, D, LANES>, f: F, ) -> ArrTensor<V, N, D, LANES>

Apply a function pairwise elementwise over two ArrTensors, mapping to a new tensor.

Panics

Both tensors, self and other, must have the same shape or a panic will occur.

Source

impl<T, const N: usize, const D: usize, const LANES: usize> ArrTensor<T, N, D, LANES>

pub fn matmul<const M: usize, const O: usize>( &self, rhs: &ArrTensor<T, M, D, LANES>, out: &mut ArrTensor<T, O, D, LANES>, )

Batched matmul over arbitrary leading batch dims: Contracts last dim of self with second-last dim of rhs.

Matrix multiplication cannot be performed on ArrTensors when the dimensions exceed MAX_STATIC_RANK. To bypass this limit, DynTensor can be used which is allocated on the heap.

Example
  • self shape: […, M, K]
  • rhs shape: […, K, N]
  • output shape: […, M, N]

Output buffer must be preallocated.

Panics

This method panics when if any of these conditions are not met:

  • both tensors must have at least 2 dimensions
  • each tensors’ inner and batch dimensions must match
  • the output tensor must be the correct shape

Every one of those must be true.

Source

impl<const N: usize, const D: usize> ArrTensor<f32, N, D>

pub fn simd_matmul<const M: usize, const O: usize>( &self, rhs: &ArrTensor<f32, M, D>, out: &mut ArrTensor<f32, O, D>, )

SIMD-accelerated matrix multiplication like Self::matmul.

This is purely an enhanced version of regular matrix multiplication with the addition of SIMD acceleration. Implementation and details should be found consulting that documentation, not this function.

Panics

Same preconditions as generic matmul.

Source

impl<const N: usize, const D: usize> ArrTensor<f64, N, D>

pub fn simd_matmul<const M: usize, const O: usize>( &self, rhs: &ArrTensor<f64, M, D>, out: &mut ArrTensor<f64, O, D>, )

SIMD-accelerated single-precision matrix multiplication like Self::matmul.

This is purely an enhanced version of regular matrix multiplication with the addition of SIMD acceleration. Implementation and details should be found consulting that documentation, not this function.

Panics

Same preconditions as generic matmul.

Source

impl<T, const N: usize, const D: usize, const LANES: usize> ArrTensor<T, N, D, LANES>

pub fn transpose(&self) -> Self

Transposes the tensor using a default axis permutation:

  • For 2D tensors, swaps the two axes.
  • For higher-rank tensors, reverses the axes.
Panics

Panics if D exceeds MAX_STATIC_RANK.

Example
use tensor_optim::{ArrTensor, TensorOps};

let tensor = ArrTensor::<f32, 6, 2>::with_data([2, 3], [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]);
let transposed = tensor.transpose();
assert_eq!(transposed.shape(), [3, 2]);
assert_eq!(transposed.data(), [1.0, 4.0, 2.0, 5.0, 3.0, 6.0]);
pub fn transpose_axes(&self, perm: [usize; D]) -> Self

Returns a new ArrTensor with axes permuted according to perm.

Panics
  • If perm is not a permutation of [0..D].
Example
use tensor_optim::{ArrTensor, TensorOps};

let tensor = ArrTensor::<f32, 6, 2>::with_data([2, 3], [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]);
let transposed = tensor.transpose_axes([1, 0]);
assert_eq!(transposed.shape(), [3, 2]);
assert_eq!(transposed.data(), [1.0, 4.0, 2.0, 5.0, 3.0, 6.0]);
pub fn transpose_axes_unchecked(&self, perm: [usize; D]) -> Self

Permutes the axes of self assuming a valid permutation.

This is roughly equivalent, though marginally more efficient, compared to Self::transpose_axes.